<?php
  /*
   Plugin Name: KCite
   Plugin URI: http://knowledgeblog.org/kcite-plugin
   Description: Add references and bibliography to blogposts
   Version: 1.5.1
   Author: Simon Cockell, Phillip Lord
   Author URI: http://knowledgeblog.org
   Email: knowledgeblog-discuss@knowledgeblog.org
   
   Copyright (c) 2010. Simon Cockell (s.j.cockell@newcastle.ac.uk)
   Phillip Lord (phillip.lord@newcastle.ac.uk)
   Newcastle University. 

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   
  */



class KCite{
    
  static $entrez_slug="&email=knowledgeblog-discuss%40knowledgeblog.org&tool=kcite";

  static $bibliography;
  
  // delete any transients as we are going, which deletes the cache
  static $clear_transients = false;
  
  // have we met any shortcodes, else block
  static $add_script = false;
  
  /**
   * Adds filters and hooks necessary initializiation. 
   */
  function init(){
    register_activation_hook(__FILE__, array(__CLASS__, 'refman_install'));
    
    //add bibliography to post content
    // priority 12 is lower than shortcode (11), so can assure that this runs
    // after the shortcode filter does, otherwise, it is all going to work
    // very badly. 
    add_filter('the_content', array(__CLASS__,'bibliography_filter'),12 );
    
    // add a filter to specify the sections
    add_filter('the_content', 
               array(__CLASS__, 'bibliography_section_filter'),14 );
      

    add_shortcode( "cite", 
                   array( __CLASS__, "cite_shortcode" ));

    
    //provide links to the bibliography in various formats
    //add_action('template_redirect', array(__CLASS__, 'bibliography_output'));

    //add settings menu link to sidebar
    add_action('admin_menu', array(__CLASS__, 'refman_menu'));
    //add settings link on plugin page
    add_filter('plugin_action_links', array(__CLASS__, 'refman_settings_link'), 9, 2 );

    
    add_action( 'wp_footer', 
                array( __CLASS__, 'add_script' ) );


    add_option( "kcite-cache", true );
  }

  /**
   * Adds options into data. Called on plugin activation. 
   */

  function refman_install() {
    //registers default options
    add_option('service', 'doi');
    add_option('crossref_id', null); //this is just a placeholder
  }

  /**
   * Section filter -- defines a section header
   */
  function bibliography_section_filter($content){
      
      $postid = get_the_ID();
      return 
          "<div class=\"kcite-section\" kcite-section-id=\"$postid\">
$content
</div> <!-- kcite-section $postid -->";
  }

  
  function add_script(){
      echo "<!-- Kcite Plugin Installed";

      if( !self::$add_script ){
          echo ": Disabled as there are no shortcodes-->\n";
          return;
      }
      echo "-->\n";
      
      if( get_option( "citeproc" ) ){
      
          // load enqueue the scripts
          wp_enqueue_script( "xmle4x", plugins_url( "kcite-citeproc/xmle4x.js", __FILE__ ), false, null, true );
          wp_enqueue_script( "xmldom", plugins_url( "kcite-citeproc/xmldom.js",__FILE__  ), false, null, true );
          wp_enqueue_script( "citeproc", plugins_url( "kcite-citeproc/citeproc.js",__FILE__  ), false, null, true );
          wp_enqueue_script( "jquery" );
          wp_enqueue_script( "kcite_locale_style", 
                             plugins_url( "kcite-citeproc/kcite_locale_style.js", __FILE__  ), false, null, true );
          wp_enqueue_script( "kcite", plugins_url( "kcite-citeproc/kcite.js",__FILE__  ), false, null, true );
          
          // and print them or they won't be printed because the footers already done
          wp_print_scripts( "xmle4x" );
          wp_print_scripts( "xmldom" );
          wp_print_scripts( "citeproc" );
          wp_print_scripts( "jquery" );
          wp_print_scripts( "kcite_locale_style" );
          wp_print_scripts( "kcite" );
      }
  }

  /**
   * citation short code
   */

  function cite_shortcode($atts,$content)
  {
      // we have a short code, so remember this for later
      self::$add_script = true;

      // extract attributes as local vars
      extract( shortcode_atts
               ( 
                array(
                      "source" => get_option( "service" ) 
                      ), $atts ) );
    
      // lazy instantiate bib
      if( !isset( self::$bibliography ) ){
          self::$bibliography = new Bibliography();
      }
    
      // store citation in bibliography. Replace anchor. 
      $cite = new Citation();
    
      $cite->identifier=$content;
      if( !isset( $source ) ){
          $source = get_option("service");
      }
      $cite->source=$source;
      
      if( !get_option( "citeproc" ) ){
          $anchor = self::$bibliography->add_cite( $cite );
          return 
              "<span id=\"cite_$anchor\" name=\"citation\">" .
              "<a href=\"#bib_$anchor\">[$anchor]</a></span>";
      }
      else{
          // this needs replacing with a URL
          $stub = array
              (
               "doi" => "http://dx.doi.org",
               "pubmed" => "http://www.ncbi.nlm.nih.gov/pubmed",
               "arxiv" => "http://arxiv.org/abs"
               );
          
          $url = "$stub[$source]/$content";
          $in_text = "<a href=\"$url\">$url</a>";
          $anchor = self::$bibliography->add_cite( $cite );
          return "<span class=\"kcite\" kcite-id=\"ITEM-$anchor\">($in_text)</span>\n";
      }
  }

  function bibliography_filter($content) {
      $bib_html = self::get_html_bibliography();
      // delete the bib -- or it will appear on subsequent posts
      self::$bibliography = null;

      return $content . $bib_html;
  }

  function get_html_bibliography(){
      
      // check the bib has been set, otherwise there have been no cites. 
      if( !isset( self::$bibliography ) ){ 
          return "<!-- kcite active, but no citations found -->";
      }
      
      $cites = self::$bibliography->get_cites();
      
      // // get the metadata which we are going to use for the bibliography.
      $cites = self::resolve_metadata($cites);
      
      if( !get_option( "citeproc" ) ){
          
          // synthesize the "get the bib" link
          $permalink = get_permalink();
 
          // it would make more sense to operate this over the citation
          // objects rather than the JSON, but this was coded as a quick way
          // of rendering after citeproc-js turned out to be a lot of work.
          // Now this is meant to be a fall back, and it will need recoding at
          // some point, but it's not urgent.
          $bibliography = 
              self::build_bibliography
              ( self::citation_to_citeproc( $cites ) );
          
          return $bibliography;
      }
      

      $cite_number = count( $cites );
      $cite_json = self::citation_to_citeproc_json( $cites );
      $url = plugins_url("kcite-citeproc",__FILE__);
      $postid = get_the_ID();
      // citeproc rendering...
      $script = <<< EOT


<p>Bibliography
      <div class="kcite-bibliography"></div>
</p>


<script type="text/javascript">
      var kcite_citation_data;
      if( kcite_citation_data == undefined ){
          kcite_citation_data = [];
      }
      kcite_citation_data[ $postid ] = $cite_json;
</script>


EOT;

       return $script;
  }

  /**
   * Builds the HTML for the bibliography. 
   *
   * Array contains the citation objects as JSON translated into a PhP array.
   *
   */
  private function build_bibliography($pub_array) {

      $i = 1;
      $bib_string = "<h2>References</h2>
    <ol>
    ";
      $temp = strval( $pub_array );
      
      foreach ($pub_array as $pub) {
          
          $anchor = "<a name='bib_$i'></a>";
          
          if( array_key_exists( "timeout", $pub ) ){
              if( array_key_exists( "source", $pub ) ){
                      $source = $pub[ "source" ] . ":";
              }
              else{
                  $source = "";
              }
                          
              $bib_string .= 
                  "<li>$anchor" . $source . $pub["identifier"] .
                  " <i>(Timed out)</i></li>\n";
              $i++;
              continue;                 
          }
          
          // we haven't been able to resolve anything
          // if( array_key_exists( "identifier", $pub ) && 
          //     array_key_exists( "source", $pub ) ){
          //     $bib_string .= 
          //         "<li>$anchor" . $pub["source"] . ": " 
          //         . $pub["identifier"] . "</li>";

          //     $i++;
          //     continue;
          // }

          if (array_key_exists( "error", $pub ) && $pub['error']){
              
              //sufficient missing to assume no publication retrieved...
              if (array_key_exists( "DOI", $pub ) && $pub['DOI']) {
                  $bib_string .= "<li>$anchor<a href='http://dx.doi.org/".
                      $pub['DOI']."'>DOI:".$pub['DOI'].
                      "</a> <i>(KCite cannot find metadata for this paper)</i></li>\n";
              }
              if (array_key_exists( "PMID", $pub ) && $pub['PMID']) {
                  $bib_string .= "<li>$anchor<a href='http://www.ncbi.nlm.nih.gov/pubmed/"
                      .$pub['PMID']."'>PMID:".$pub['DOI'].
                      "</a> <i>(KCite cannot find metadata for this paper)</i></li>\n";
              }
          }
          else {
              
              $bib_string .= "<li>$anchor
";
              $author_count = 1;
              $author_total = count($pub['author']);
              foreach ($pub['author'] as $author) {
                  //get author initials
                  $firsts = $author['given'];
                  $words = explode(' ', $firsts);
                  $initials = "";
                  foreach ($words as $word) {
                      $initials .= strtoupper(substr($word,0,1)).".";
                  }
                  $initials;
                  $bib_string .= $initials." ".$author['family'].", ";
                  if ($author_count == ($author_total - 1)) {
                      $bib_string .= "and ";
                  }
                  $author_count++;
              }
              if ($pub['title']) {
                  $bib_string .= '"'.$pub['title'].'"';
              }
              if ($pub['container-title']) {
                  $bib_string .= ', <i>'.$pub['container-title'].'</i>';
              }
              if (array_key_exists("volume", $pub)){
                  $bib_string .= ', vol. '.$pub['volume'];
              }
              
              if (array_key_exists("issued", $pub)){
                  if(array_key_exists("date-parts", $pub["issued"])){
                      if(array_key_exists( 0, $pub["issued"]["date-parts"])){
                          if(array_key_exists( 0, $pub["issued"]["date-parts"][0])){
                              $bib_string .= ', '.$pub['issued']['date-parts'][0][0];
                          }
                      }
                  }
              }
              if (array_key_exists("page", $pub) ) {
                  $bib_string .= ', pp. '.$pub['page'];
              }
              if (array_key_exists("DOI", $pub) ) {
                  $bib_string .= '. <a href="http://dx.doi.org/'.
                      $pub['DOI'].'" target="_blank" title="'
                      .$pub['title'].'">DOI</a>';
              }
              $bib_string .= ".
</li>
";
          }
          $i++;
      }
      $bib_string .= "</ol>
";

      if( self::$bibliography->contains_timeout ){
          $bib_string .= <<<EOT
<p><a href="http://knowledgeblog.org/kcite-plugin/">Kcite</a> was unable to 
retrieve citation information for all the references, due to a timeout. This
is done to prevent an excessive number of requests to the services providing
this information. More references should appear on subsequent page views</p>

EOT;
      }

      return $bib_string;
  }
  
  
  /**
   * Expands citation objects to include full details. 
   * This can be used to build the JSON. 
   */
  private function resolve_metadata($cites) {
      
      $start_time = time();
      
      foreach ($cites as $cite) {
          
          // print( "Testing time: " . (time() - $start_time) . "\n" );
          
          // check whether this is all taking too long
          if( time() - $start_time > get_option( 'timeout', 5 ) ){
              $cite->error = true;
              $cite->timeout = true;
              self::$bibliography->contains_timeout = true;
              continue;
          }
          
          if ($cite->source == 'doi') {
              // should work on datacite or crossref lookup
              $cite = self::dx_doi_lookup($cite);
              
              //failover to pubmed
              if (!$cite->resolved) {
                  $cite = self::pubmed_doi_lookup($cite);
              }

              if($cite->resolved && $cite->resolved_from=="crossref" ){
                  $cite = self::get_crossref_metadata($cite);
                  continue;
              }
              
              if($cite->resolved && $cite->resolved_from=="datacite" ){
                  $cite = self::parse_xml($cite);
                  $cite = self::get_datacite_metadata($cite);
                  continue;
              }
               
              if( $cite->resolved && $cite->resolved_from="pubmed" ){
                  $cite = self::parse_xml($cite);
                  $cite = self::get_pubmed_metadata($cite);
                  continue;
              }
              
              // doi we can't find. 
              $cite->error = true;
              continue;
          }
          
          if ($cite->source == 'pubmed') {
              $cite = self::pubmed_id_lookup($cite);
              
              if (!$cite->resolved) {
                  $cite->error = true;
                  continue;
              }
              
              
              $cite = self::parse_xml($cite);
              $cite = self::get_pubmed_metadata($cite);
              continue;
          }

          if( $cite->source == "arxiv"){
              $cite = self::arxiv_id_lookup($cite);
              if( !$cite->resolved){
                  $cite->error = true;
                  continue;
              }
              $cite = self::parse_xml($cite);
              $cite = self::get_arxiv_metadata($cite);
              continue;
          }

          
          // if we don't recognise the type then we have an error
          $cite->error = true;
      }
      
      return $cites;
  }
  

  /**
   * Attempt to resolve metadata for a citation object
   * @param string $pub_doi A doi representing a reference
   * @return 
   */
  private function dx_doi_lookup($cite) {

      // slug includes JSON so we ignore XML in the database from previous
      // incarnations of kcite.
      $crossref_trans_slug = "crossref-doi-json" . $cite->identifier;
      $datacite_trans_slug = "datacite-doi-xml" . $cite->identifier;
      
      // debug code -- blitz transients in the database
      if( self::$clear_transients ){
          delete_transient( $crossref_trans_slug );
          delete_transient( $datacite_trans_slug );
      }
      
      if(get_option("kcite-cache") && $json = get_transient($crossref_trans_slug)){
          $cite->resolved = true;
          $cite->resolution_source=$json;
          $cite->resolved_from="crossref";
          return $cite;
      }

      if(get_option("kcite-cache") && $xml = get_transient($datacite_trans_slug)){
          $cite->resolved = true;
          $cite->resolution_source=$xml;
          $cite->resolved_from="datacite";
          return $cite;
      }
          
          
      $url = "http://dx.doi.org/{$cite->identifier}";
          
      // get the metadata with negotiation
      $ch = curl_init();
      curl_setopt ($ch, CURLOPT_URL, $url );
      curl_setopt ($ch, CURLOPT_RETURNTRANSFER, true );
      curl_setopt ($ch, CURLOPT_FOLLOWLOCATION, true );
      // the order here is important, as both datacite and crossrefs content negotiation is broken. 
      // crossref only return the highest match, but do check other content
      // types. So, should return json. Datacite is broken, so only return the first
      // content type, which should be XML.
      curl_setopt ($ch, CURLOPT_HTTPHEADER,
                   array (
                          "Accept: application/x-datacite+xml;q=0.9, application/citeproc+json;q=1.0"
                          ));
      
      // debug
      //$fh = fopen('/tmp/curl.log', 'w'); 
      //curl_setopt($ch, CURLOPT_STDERR, $fh );
      //curl_setopt($ch, CURLOPT_VERBOSE, true );
      
      $response = curl_exec ($ch);
      $status = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
      $contenttype = curl_getinfo($ch, CURLINFO_CONTENT_TYPE);
      
      // it's probably not a DOI at all. Need to check some more here. 
      if( curl_errno( $ch ) == 404 ){
          curl_close($ch);
          return $cite;
      }            

      curl_close ($ch);
                

      if( $contenttype == "application/citeproc+json" ){
          // crossref DOI
          $cite->resolved = true;
          $cite->resolution_source=$response;
          $cite->resolved_from="crossref";
          
          // set the transient with a slug so we can distinguish from datacite
          set_transient( $crossref_trans_slug, $response, 60*60*24*7 );

          return $cite;
      }
              
      if( $contenttype == "application/x-datacite+xml" ){
          //datacite DOI
          $cite->resolved = true;
          $cite->resolution_source=$response;
          $cite->resolved_from="datacite";
          
          // set the transient with a slug so we can distinguish from crossref
          set_transient( $datacite_trans_slug, $response, 60*60*24*7 );
          return $cite;
      }

    return $cite;
  }

  /**
   * Look up DOI on pubmed
   * @param string $cite A doi representing a reference
   * @return resolved (or not) citation object
   */

  private function pubmed_doi_lookup($cite) {
      
      $trans_slug = "pubmed-doi-to-pubmed" . $cite->identifier;

      // debug code -- blitz transients in the database
      if( self::$clear_transients ){
          delete_transient( $trans_slug );
      }
    
      if (false === (get_option( "cache" ) 
                     && $id = get_transient( $trans_slug ))) {

          // print( "pubmed_doi lookup:$trans_slug " . date( "H:i:s", time() ) . "\n" );
          // a free text search for the DOI!
          $search = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/esearch.fcgi?" . 
              self::$entrez_slug . "&db=pubmed&retmax=1&term="
              .$cite->identifier;

          $search_xml = file_get_contents($search, 0);
          
          if (preg_match('/PhraseNotFound/', $search_xml)) {
              return $cite;
          }
      
          // now parse out the DOI
          $search_obj =  new SimpleXMLElement($search_xml);
          $idlist = $search_obj->IdList;
          $id = $idlist->Id;
          
          set_transient( $trans_slug, strval( $id ), 60*60*24*7 );
      }
      
      // now do the pubmed_id_lookup!
      // this is not ideal as we are dumping the original 
      $cite->identifier = $id;
      $cite->source = "pubmed";
      
      return self::pubmed_id_lookup($cite);
  }

  
  /**
   * Look up pubmed ID on pubmed
   * @param Citation the citation to resolve
   * @return null if DOI does not resolve, or raw pubmed XML
   */
  private function pubmed_id_lookup($cite) {

      
      $trans_slug = "pubmed-id" . $cite->identifier;
      
      // debug code -- blitz transients in the database
      if( self::$clear_transients ){
          delete_transient( $trans_slug );
      }
    
      if (false === (get_option( "kcite-cache") && $xml = get_transient( $trans_slug ))) {

          // print( "pubmed_id lookup: $trans_slug" . date( "H:i:s", time() ) . "\n" );

          $fetch = "http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?" 
              . self::$entrez_slug . "&db=pubmed&retmode=xml&id="
              .$cite->identifier;
          $xml = file_get_contents($fetch, 0);
          if (preg_match('/(Error|ERROR)>/', $xml)) {
              //handles fetch failure
              return $cite;
          }
          set_transient( $trans_slug, $xml, 60*60*24*7 );
      }
      
      $cite->resolved = true;
      $cite->resolution_source = $xml;
      $cite->resolved_from = "pubmed";
      return $cite;
  }


  private function arxiv_id_lookup($cite){
      $trans_slug = "arxiv-id" . $cite->identifier;
      
      if( self::$clear_transients ){
          delete_transient( $trans_slug );
      }
      if( false === (get_option( "kcite-cache")
                     && $xml = get_transient( $trans_slug ))) {
      
          $fetch = "http://export.arxiv.org/oai2?verb=GetRecord&identifier=oai:arXiv.org:" . $cite->identifier . "&metadataPrefix=arXiv";

          $xml = file_get_contents( $fetch, 0 );
          // TODO failure handling here 
          set_transient( $trans_slug, $xml, 60*60*24*7 );
      }
      
      $cite->resolved = true;
      $cite->resolution_source = $xml;
      $cite->resolved_from = "arxiv";
      
      return $cite;
  }

  /**
   * Parses XML in a citation object into a PhP array
   * @param Citation object
   * @return Citation with parsedXML now containing SimpleXMLElement object
   */
  private function parse_xml($cite) {
      $cite->parsedXML = new SimpleXMLElement( $cite->resolution_source );
      return $cite;
  }
  
  /**
   * Badly named method, restful API showing just the JSON object for the reference list. 
   * This is not functional at the moment. 
   * 
   */
  // function bibliography_output() {
  //   global $post;
  //   $uri = self::get_requested_uri();
  //   if ($uri[0] == 'json') {
  //       //render the json here
  //       $this_post = get_post($post->ID, ARRAY_A);
  //       $post_content = $this_post['post_content'];
  //       $dois = self::get_cites($post_content);
  //       $metadata = array();
  //       $metadata = self::get_arrays($dois[1]);
  //       $json = self::metadata_to_json($metadata);
  //       echo $json;
  //       exit;
  //   }
  //   elseif ($uri[0] == 'bib') {
  //       //render bibtex here
  //       exit;
  //   }
  //   elseif ($uri[0] == 'ris') {
  //       //render ris here
  //       exit; //prevents rest of page rendering
  //   }
  // }

  
  /**
   * @param array of Citation objects
   * @return php array ready for JSON encoding
   */
  private function citation_to_citeproc($cites){
      
      $citep = array();
      
      $item_number = 1;
      $cite_length = count($cites);
      
      foreach ($cites as $cite) {
          $item_string = "ITEM-".$item_number++;
          
          if( $cite->json ){
              
              $item = $cite->json;
              
              // add a few bits of additional metadata
              $item["source"] = $cite->source;
              $item["identifier"] = $cite->identifier;
              $item["resolved"] = $cite->resolved;
              $item["id"] = "$item_string";
              

              // we finished!
              $citep[ $item_string ] = $item;
              continue;
          }

          
          $item = array();
          
          $item["source"] = $cite->source;
          $item["identifier"] = $cite->identifier;
          $item["resolved"] = $cite->resolved;
          
          // timed out overall, so don't have the metadata
          if( $cite->timeout ){
              $item["timeout"] = true;
              $citep[ $item_string ] = $item;
              continue;
          }
          
          // there was an error of some sort (normally no metadata)
          if( $cite->error ){
              $item["error"] = true;
              $citep[ $item_string ] = $item;
              continue;
          }
          
          
          // just didn't resolve
          if( !$cite->resolved ){
              // did we resolve or not?
              $citep[ $item_string ] = $item;          
              continue;
              
          }
          
          // normal condition
          // stick this on both sides for the hell of it
          $item[ "id" ] = "$item_string";
          $item[ "title" ] = $cite->title;
          
          $authors = array();
          
          foreach ($cite->authors as $author) {
              
              $auth = array();
              $auth["family"] = $author[ "surname" ];
              $auth["given"]  = $author[ "given_name" ];
              $authors[] = $auth;
          }
          $item[ "author" ] = $authors;
          
          $item[ "container-title" ] = $cite->journal_title;
          
          // dates -- only if we have year
          if( $cite->pub_date[ 'year' ] ){
              $issued = array();
              $date_parts = array();
              $date_parts[] = (int)$cite->pub_date[ 'year' ];
              // month and day if existing or nothing
              
              if(array_key_exists( "month", $cite->pub_date)){
                  $date_parts[] = (int)$cite->pub_date[ 'month' ];
              }
              if(array_key_exists( "day", $cite->pub_date)){
                  $date_parts[] = (int)$cite->pub_date[ 'day' ];
              }
              
              $issued[ "date-parts" ] = array( $date_parts );
              $item[ "issued" ] = $issued;
          }
          
          if($cite->first_page){
              $item[ "page" ] = 
                  $cite->first_page . "-" . $cite->last_page;
          }
          
          if( $cite->volume ){
              $item[ "volume" ] = $cite->volume;
          }
          
          if( $cite->issue ){
              $item[ "issue" ] = $cite->issue;
          }
          
          if( $cite->reported_doi ){
              $item[ "DOI" ] = $cite->reported_doi;
          }
          
          $item[ "type" ] = "article-journal";
          
          if( $cite->url ){
              $item["URL"] = $cite->url;
          }


          $citep[ $item_string ] = $item;
          
      }

      return $citep;
  }
  
  /**
   * @param array of Citation objects
   * @return JSON encoded string for citeproc
   */
  private function citation_to_citeproc_json( $cites ){
      
      
      $citep = self::citation_to_citeproc( $cites );

      // crude hack --- http://bugs.php.net/bug.php?id=49366 PHP escapes all /
      // which I think is going to stop things later on (although I haven't
      // checked this yet. JSON_UNESCAPED_SLASHES is the way forward when it
      // gets into PHP.
      return str_replace('\\/', '/', json_encode( $citep ) );
  }

  /**
   * @param Citation $cite crossref resolved citation
   * @return Citation with metadata extracted
   */
   private function get_crossref_metadata($cite) {
       
       $json_decoded = json_decode( $cite->resolution_source, true );

       // crossref returns both url and raw DOI. We don't need the later, so delete it. 
       unset( $json_decoded[ "DOI" ] );
       $cite->json = $json_decoded;


       return $cite;
   }

   private function get_datacite_metadata($cite){
       
       $article = $cite->parsedXML;
       $namespaceN = $article->getNamespaces();
       
       // nasty name space hack
       // datacite returns more than one form, but with different name spaces
       // which breaks the xpath, even they are the same for my purposes. 
       $kn = "";
       if( $namespaceN[ "" ] == "http://datacite.org/schema/kernel-2.2" ){
           $kn = "kn:";
           $article->registerXpathNamespace( "kn", "http://datacite.org/schema/kernel-2.2" );
       }
       
       if( $namespaceN[ "" ] == null ){
           // kernel 2.0 -- no namespace
           // so do nothing.
       }
       
       $journalN = $article->xpath( "//${kn}publisher"); 
       // we get lots of newlines without trim
       $cite->journal_title = trim( (string)$journalN[ 0 ] );

       // datacite can give multiple titles, it appear
       $titleN = $article->xpath( "//${kn}title" );
       $cite->title = trim( (string)$titleN[ 0 ] );

       $authorN = $article->xpath( "//${kn}creators/${kn}creator/${kn}creatorName" );

       foreach( $authorN as $author ){
           // this is not the most high tech name parsing ever. 
           
           // names usualy come as Smith, J
           list( $last, $first ) = 
               explode( ",", trim((string)$author) );
           
           // but sometimes are consortia names
           if( $last == null ){
               $last = $author;
               $first = "";
           }                                

           $newauthor = array();
           $newauthor['surname'] = $last;
           $newauthor['given_name'] = $first;
           
           $cite->authors[] = $newauthor;
       }

       $yearN = $article->xpath( "//${kn}publicationYear" );
       $cite->pub_date[ 'year' ] = (string)$yearN[ 0 ];

       $cite->url = "http://dx.doi.org/" . $cite->identifier;
   }

   /**
   * @param string $article returns metadata object from SimpleXMLElement
   * @return metadata associative array
   */
   private function get_pubmed_metadata($cite) {

      // dump the XML
      //print( "<br> Resolved Data for pubmed:$cite->identifer<br>$cite->resolution_source<br>" );

      // actually an article set -- so this code should generalize okay
      $article = $cite->parsedXML;
      

      $issueN = $article->xpath( "//Article/Journal/JournalIssue/Issue" );
      if( count( $issueN ) > 0 ){
          $cite->issue = (string)$issueN[ 0 ];
      }
      
      $journal_titleN = $article->xpath( "//Journal/Title" );
      if( count( $journal_titleN ) > 0 ){
          $cite->journal_title = (string)$journal_titleN[ 0 ];
      }
      
      $volN = $article->xpath( "//Journal/Volume" );
      if( count( $volN ) > 0 ){
          $cite->volume = (string)$volN[ 0 ];
      }

      $abbrN = $article->xpath( "//Journal/ISOAbbreviation" );
      if( count( $abbrN ) > 0 ){
          $cite->abbrv_title = (string)$abbrN[ 0 ];
      }
      
      $artN = $article->xpath( "//ArticleTitle" );
      if( count( $artN ) > 0 ){
          $cite->title = (string)$artN[ 0 ];
      }

      $authN = $article->xpath( "//AuthorList/Author" );
      foreach($authN as $author){
            $newauthor = array();
            $newauthor['given_name'] = (string)$author->ForeName;
            $newauthor['surname'] = (string)$author->LastName;
            
            $cite->authors[] = $newauthor;
      }
      
      $artDN = $article->xpath( "//ArticleDate" );

      // TODO PWL this bit is failing -- days are not always reported
      // Untested -- handle missing date parts later. 
      if( count( $artDN ) == 0 ){
          $artDN = $article->xpath( "//JournalIssue/PubDate" );
      }
      
      if( count( $artDN ) > 0 ){
          $cite->pub_date[ 'month' ] = (string)$artDN[ 0 ]->Month;
          $cite->pub_date[ 'day' ] = (string)$artDN[ 0 ]->Day;
          $cite->pub_date[ 'year' ] = (string)$artDN[ 0 ]->Year;
      }
      
      $elocN = $article->xpath( "//ELocationID" );
      if( count( $elocN ) > 0 ){
          $cite->reported_doi = (string)$elocN[ 0 ];
      }
      
      $cite->url = "http://www.ncbi.nlm.nih.gov/pubmed/{$cite->identifier}";

      return $cite;
  }
  

   private function get_arxiv_metadata($cite){
       
       $article = $cite->parsedXML;
       $article->registerXpathNamespace( "ar", "http://arxiv.org/OAI/arXiv/" );
       $article->registerXpathNamespace( "oai", "http://www.openarchives.org/OAI/2.0/" );
       $cite->journal_title = "arXiv";
       
       $titleN = $article->xpath( "//ar:title" );
       $cite->title = (string)$titleN[ 0 ];

       $dateN = $article->xpath( "//ar:created" );
       $rawdate = (string)$dateN[ 0 ];
       
       $cite->pub_date['month'] = substr( $rawdate, 5, 2 );
       $cite->pub_date['day'] = substr( $rawdate, 8, 2 );
       $cite->pub_date['year'] = substr( $rawdate, 0, 4 );
       
       $authorN = $article->xpath( "//ar:author" );
       
       foreach( $authorN as $author ){
           $newauthor = array();
           $newauthor['surname'] = (string)$author->keyname;
           $newauthor['given_name'] = (string)$author->forenames;
           $cite->authors[] = $newauthor;
       }

       $cite->url = "http://arxiv.org/abs/" . $cite->identifier;
         
       return $cite;
   }


  /**
   * Fetches the URI that the user requested to work out output format. 
   */
  private function get_requested_uri() {
    $requesturi = $_SERVER['REQUEST_URI'];
    preg_match('#\/(.*)\/bib\.(bib|ris|json)$#', $requesturi, $matches);
    //matches[1] is post (with extraneous paths maybe)
    $uri = null;
    if ($matches) { 
        $uri = array();
        $uri[0] = $matches[2];
    }
    return $uri;
  }
  
  /**
   * add a link to settings on the plugin management page
   */ 
  function refman_settings_link( $links, $file ) {
    if ($file == 'kcite/kcite.php' && function_exists('admin_url')) {
        $settings_link = '<a href="' .admin_url('options-general.php?page=kcite.php').'">'. __('Settings') . '</a>';
        array_unshift($links, $settings_link);
    }
    return $links;
  }

  /** 
   * Link from Settings menu widget to options page. 
   */
  function refman_menu() {
    add_options_page('KCite Plugin Options', 'KCite Plugin', 'manage_options', 'kcite', array(__CLASS__, 'refman_plugin_options'));
  }
  
  /**
   * Prints options form and process it. 
   */
  function refman_plugin_options() {
      if (!current_user_can('manage_options'))  {
        wp_die( __('You do not have sufficient permissions to access this page.') );
      }
      echo '<div class="wrap" id="refman-options">
<h2>KCite Plugin Options</h2>
';
    if ($_POST['refman_hidden'] == 'Y') {
        //process form
        if ($_POST['service'] != get_option('service')) {
            update_option('service', $_POST['service']);
        }
        if ($_POST['crossref_id']) {
            if(eregi("^[_a-z0-9-]+(\.[_a-z0-9-]+)*@[a-z0-9-]+(\.[a-z0-9-]+)*(\.[a-z]{2,3})$", $_POST['crossref_id'])) {
                if ($_POST['crossref_id'] != get_option('crossref_id')) {
                    update_option('crossref_id', $_POST['crossref_id']);
                }
            }
            else {
                echo "<div style='background-color:rgb(255,168,206);width:80%;padding:4px;border-style:solid;border-width:1px;' id='kcite-options-error'>
                Warning - The CrossRef user ID should be a valid email address.
                </div>
                ";
            }
        }
        if ($_POST['citeproc']){
            if( $_POST['citeproc'] == "True"){
                update_option( 'citeproc', true );
            }
            else{
                update_option( 'citeproc', false );
            }
        }

        if ($_POST['kcite-cache']){
            if( $_POST['kcite-cache'] == "True"){
                update_option( 'kcite-cache', true );
            }
            else{
                update_option( 'kcite-cache', false );
            }
        }

        if( $_POST['timeout']){
            update_option( 'timeout', $_POST['timeout'] );
        }
        echo '<p><i>Options updated</i></p>';   
    }

    

?>   
      <form id="refman" name="refman" action="" method='POST'>
      <input type="hidden" name="refman_hidden" value="Y">
      <table class="form-table">
      <tr valign="middle">
      <th scope="row">Default Identifier Type<br/><font size='-2'>Which type of identifier would you like to use as the default?</font></th>
      <td><select name='service'>
        <option value='doi' <?php if (get_option('service') == 'doi') echo 'SELECTED'; ?>>DOI</option>
        <option value='pubmed' <?php if (get_option('service') == 'pubmed') echo 'SELECTED'; ?>>PubMed</option>
      </select>
      </td>
      </tr>
      <th scope="row">CrossRef User ID<br/><font size='-2'>Enter an e-mail address that has been <a href='http://www.crossref.org/requestaccount/'>registered with the CrossRef API</a>.</th>
      <td><input type='text' name='crossref_id' class='regular-text code' value='<?php echo get_option('crossref_id'); ?>'></td>
      </tr>
      <th scope="row">Use Citeproc rendering<br/><font size='-2'>Do you wish to build the bibliography on the client?</font></th>
      <td><select name='citeproc'>
        <option value='True' <?php if (get_option('citeproc')) echo 'SELECTED'; ?>>True</option>
        <option value='False' <?php if (!get_option('citeproc')) echo 'SELECTED'; ?>>False</option>
      </select>
      </td>
      <tr>
      <th scope="row">Reference timeout<br/><font size='-2'>For how long should kcite attempt to gather bibliographic data before timing out</font></th>
      <td><input type='text' name="timeout" value='<?php echo get_option('timeout', 5) ?>'></td>

      <tr>
      <th scope="row">Cache References<br/><font size='-2'>Should kcite cache reference metadata. Set to false for debugging</font></th>
      <td><select name='kcite-cache'>
          <option value='True' <?php if (get_option('kcite-cache')) echo 'selected="true"'; ?>>True</option>
          <option value='False' <?php if (!get_option('kcite-cache')) echo 'selected="true"'; ?>>False</option>
      </select>
      </td>                                                                                                                                                                                                                      
      </table>
      <p class="submit">
      <input type="submit" class="button-primary" value="<?php _e('Save Changes') ?>" />
      </p>
      </form>
      </div>
<?php
   }

}

class Bibliography{
    // array of Citation objects
    private $cites = array();

    // did at least one reference time out during the production of this bibliography. 
    public $contains_timeout = false;

    function add_cite($citation){
        // unique check
        for( $i = 0;$i < count($this->cites);$i++ ){
            if( $this->cites[ $i ]->equals( $citation ) ){
                return $i + 1;
            }
        }
        
        // add new citation
        $this->cites[] = $citation;
        return count( $this->cites );
    }
    
    function get_cites(){
        return $this->cites;
    }
}


class Citation{
    
    // generic properties from the citation
    public $identifier;
    public $source;
    
    // have we translate the identifier into something more, the best we can.
    public $resolved = false;

    // has the translation resulted in an error
    public $error = false;
    
    // have we failed to retrieve this of time out or request limit
    public $timeout = false;
    
    // raw resolved data, in whatever format it comes where ever!
    public $resolution_source;
    // the where ever in the last line
    public $resolved_from;
    
    // parsed XML data as SimpleXMLElement
    public $parsedXML;
    
    // metadata represented as JSON
    public $json;
    
    // citation metadata
    public $authors = array();
    public $journal_title;
    public $abbrv_title;
    // three up array, month, day, year
    public $pub_date;
    public $volume;
    public $title;
    public $first_page;
    public $last_page;
    public $reported_doi;
    public $resource;
    public $issue;
    public $url;

    function equals($citation){
        return $this->identifier == $citation->identifier &&
            $this->source == $citation->source;
    }
}

KCite::init();

?>
